//
//  SnapCameraVC.m
//  Snap!
//
//  Created by Brandon Schlenker on 6/14/11.
//  Copyright 2011 Brandon Schlenker. All rights reserved.
//

#import "SnapCameraVC.h"

enum IrisFilterType {
    IrisFilterTypeNone = 0,
    IrisFitlerTypeBW = 1,
    IrisFilterTypeSepia = 2
};

@interface SnapCameraVC () 
- (NSImage *)imageWithCurrentFrame;
- (CIImage *)flipImage:(CIImage *)i;
- (CIImage *)applyFilterWithImage:(CIImage *)i;
- (void)snapPicture:(id)sender;
@end

@implementation SnapCameraVC

@synthesize busy, delegate, captureView, accountButton, prefButton, loadingIndicator, DBApi;

- (id)initWithNibName:(NSString *)nibNameOrNil bundle:(NSBundle *)nibBundleOrNil
{
    self = [super initWithNibName:nibNameOrNil bundle:nibBundleOrNil];
    if (self) {
        [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(didLogin:) name:@"DAILYBOOTH_DID_LOGIN" object:nil];
        
        [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(didLogout:) name:@"DAILYBOOTH_DID_LOGOUT" object:nil];
        
         [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(selectedNewCamera:) name:@"IRIS_SELECTED_NEW_CAMERA" object:nil];
    
    }
    
    return self;
}

- (void)awakeFromNib
{
    if ([DBApi isAuthenticated])
    {
        NSDictionary *userData = [[NSUserDefaults standardUserDefaults] dictionaryForKey:@"oauth_info"];
        accountButton.title = [[userData objectForKey:@"user"] objectForKey:@"username"];
    }
    [self setupLayerStyling];
}

- (void)didLogin:(NSNotification *)notif
{
    NSDictionary *d = [[NSUserDefaults standardUserDefaults] objectForKey:@"oauth_info"];
    NSString *user = [[d objectForKey:@"user"] objectForKey:@"username"];
    accountButton.title = user;
}

- (void)didLogout:(NSNotification *)notif
{
    accountButton.title = @"";
}

- (void)selectedNewCamera:(NSNotification *)notif
{
    [self stopCamera];
    [self startCamera];
}

- (IBAction)openDailyBoothSite:(id)sender
{
    NSDictionary *userData = [[NSUserDefaults standardUserDefaults] dictionaryForKey:@"oauth_info"];
//    NSLog(@"%@", userData);
    if (userData)
    {
        NSString *s = [NSString stringWithFormat:@"http://dailybooth.com/%@", [[userData objectForKey:@"user"]objectForKey:@"username"]];
        
        NSURL *url = [NSURL URLWithString:s];
        [[NSWorkspace sharedWorkspace] openURL:url]; 
    }
    else
    {
        [DBApi authenticate];
    }
}

- (IBAction)openPrefs:(id)sender
{
    [delegate openPreferences:sender];
}

- (void)setupLayerStyling
{
    CALayer *l;
    l = [self.captureView layer];
    [l setCornerRadius:3.0f];
    [l setMasksToBounds:YES];
    [l setBorderWidth:0.75f];
    NSColor *color = [NSColor colorWithCalibratedWhite:0.0 alpha:0.5];
    [l setBorderColor:[color CGColor]];
}

- (void)startCamera
{
    [NSAnimationContext beginGrouping];
    [[NSAnimationContext currentContext] setDuration:0.25];
    [[self.prefButton animator] setAlphaValue:1];
    [[self.accountButton animator] setAlphaValue:1];
    [NSAnimationContext endGrouping];
    
    if (self.captureView.image == nil)
    {
        [loadingIndicator setHidden:NO];
        [loadingIndicator startAnimation:self];
    }
    else
    {
        [loadingIndicator setHidden:YES];
    }
    if (!cameraViewTimer)
    {
        cameraViewTimer = [NSTimer scheduledTimerWithTimeInterval:.01 target:self selector:@selector(switchImage:) userInfo:nil repeats:YES];
    }
    if (self.busy)
        return;
    
    session = [[QTCaptureSession alloc] init];
    
    NSString *currentSelectedCamera = [[NSUserDefaults standardUserDefaults] stringForKey:@"IRIS_SELECTED_CAMERA"];
    
    if (currentSelectedCamera)
    {
        device = [QTCaptureDevice deviceWithUniqueID:currentSelectedCamera];
    }
    else
    {
        device = [QTCaptureDevice defaultInputDeviceWithMediaType: QTMediaTypeVideo];
    }
    NSDictionary *attributes = [device deviceAttributes];
    //NSLog(@"camera: %@", attributes);
    
    [self.captureView setDelegate:self];
    qtError = nil;
    
    _frameRecieved = NO;
	self.busy = YES;
	
	if ([device open: &qtError]) {
		cameraDeviceInput = [[QTCaptureDeviceInput alloc] initWithDevice: device];
        
		if ([session addInput: cameraDeviceInput error: &qtError]) {
			cameraVideoOutput = [[QTCaptureDecompressedVideoOutput alloc] init];
            
            
			if ([session addOutput: cameraVideoOutput error: &qtError]) {
				
				
				[cameraVideoOutput setDelegate: self];
				[session startRunning];
				do {
					// if we just sleep here, we'll never get a frame
					[NSApp nextEventMatchingMask: NSAnyEventMask untilDate: [NSDate dateWithTimeIntervalSinceNow: 0.1] inMode: NSDefaultRunLoopMode dequeue: YES];
				} while (!_frameRecieved);
				
            }
			
		}
		
	}
    
	if (qtError != nil)		// bummer
		[NSAlert alertWithError: qtError];
}

- (void)switchImage:(id)sender
{
    NSImage *i = [self imageWithCurrentFrame];
    if (i)
    {
        [captureView setImage:i];
        [loadingIndicator stopAnimation:self];
    }
}

- (void)stopCamera
{
    [NSThread cancelPreviousPerformRequestsWithTarget:self selector:@selector(startCamera) object:nil];

    if (!self.busy)
        return;
    
    
    [loadingIndicator stopAnimation:self];
    
    if (session)
    {
        [session stopRunning];
        [cameraVideoOutput release];
        [session removeOutput:cameraVideoOutput];
        [session removeInput:cameraDeviceInput];
        [cameraDeviceInput release];
        [device close];
        session = nil;
        [session release];
    }
    self.busy = NO;
    _currentImageBuffer = nil;
  //  [self.captureView setImage:nil];
}


- (void)captureViewDidClick:(NSImageView *)view
{
    //  [self grabFrame:self];
    if (!cameraSnapTimer)
    {
        time = 3;
        captureView.hoverText.stringValue = [NSString stringWithFormat:@"                               %i", time];
        cameraSnapTimer = [NSTimer scheduledTimerWithTimeInterval:1 target:self selector:@selector(snapPicture:) userInfo:nil repeats:YES];
        [captureView setHoverAlwaysVisible:YES];
    }
    else
    {
        time = 1;
        [self snapPicture:nil];  
    }
}


- (void)captureViewDidDoubleClick:(NSImageView *)view
{
    if (cameraSnapTimer)
    {
        time = 1;
        [self snapPicture:nil];
        [captureView setHoverAlwaysVisible:NO];
    }
}

- (void)snapPicture:(NSTimer *)timer
{
    
    time = time - 1;
    captureView.hoverText.stringValue = [NSString stringWithFormat:@"                               %i", time];
    
    if (time == 0)
    {
        [self grabFrame:self];

        if (cameraViewTimer)
        {
            [cameraViewTimer invalidate];
            cameraViewTimer = nil;
        }
        
        if (cameraSnapTimer)
        {
            [cameraSnapTimer invalidate];
            cameraSnapTimer = nil;
        }
        time = 3;
    }
}

- (void)grabFrame:(id)sender;
{
    NSImage			*grabbedFrame;
    
    NSDateFormatter *dateFormat = [[NSDateFormatter alloc] init];
    [dateFormat setDateStyle:NSDateFormatterFullStyle];
    [dateFormat setDateFormat:@"yyyy-MM-dd 'at' HH.mm.ss"];
    NSString *date = [dateFormat stringFromDate:[NSDate date]];
    
    
    NSString *path = [NSString stringWithFormat:@"~/Pictures/Iris %@.png", date];
    path = [path stringByExpandingTildeInPath];
    
    if ((grabbedFrame = [self imageWithCurrentFrame]) != nil)
    {
        BOOL prefSaveImages = [[NSUserDefaults standardUserDefaults] boolForKey:@"prefSaveImages"];
        
        if (prefSaveImages)
        {
            [[[NSBitmapImageRep imageRepWithData: [grabbedFrame TIFFRepresentation]] representationUsingType: NSPNGFileType properties: nil] writeToFile: path atomically: YES];
        }
        [captureView setHoverAlwaysVisible:NO];
        [captureView mouseExited:nil];

        [NSAnimationContext beginGrouping];
        [[NSAnimationContext currentContext] setDuration:0.3];
        [[self.prefButton animator] setAlphaValue:0];
        [[self.accountButton animator] setAlphaValue:0];
        [NSAnimationContext endGrouping];
        
        [loadingIndicator stopAnimation:self];
        [delegate snapCameraVC:self didTakePicture:grabbedFrame];
        [self setupLayerStyling];
    }
}
- (NSImage *) imageWithCurrentFrame
{
    CVImageBufferRef	imageBuffer;
    NSImage				*image = nil;
	
	// not strictly necessary since we've probably stopped the QTCaptureSession at this point,
	// but just in case it isn't, synchronize this since the delegate need not be called in the main thread.
    @synchronized (self) {
        imageBuffer = CVBufferRetain(_currentImageBuffer);
    }
    
    if (imageBuffer != NULL) {
        CIImage *img = [CIImage imageWithCVImageBuffer: imageBuffer];
        BOOL flip = [[NSUserDefaults standardUserDefaults] boolForKey:@"prefFlipPhotos"];
        
        if (!flip)
        {
            img = [self flipImage:img];
            //            NSLog(@"flip %@",img);
        }
        
        if (currentFilterType)
        {
            img = [self applyFilterWithImage:img];
        }
        
        NSCIImageRep		*imageRep = [NSCIImageRep imageRepWithCIImage:img];
        
        NSSize imgSize = [imageRep size];
        
      /*  int max = 1280;
        if (imgSize.width > max)
        {
            float ratio = max/imgSize.width;
            float width = imgSize.width * ratio;
            float height = imgSize.height * ratio;
            
            imgSize = NSMakeSize(width, height);
        }
        imageRep.size = imgSize;*/
        
		image = [[[NSImage alloc] initWithSize: imgSize] autorelease];
        [image addRepresentation:imageRep];
        //         NSLog(@"image %@",img);
        CVBufferRelease(imageBuffer);
    }
    
	return image;
}

- (void) captureOutput:( QTCaptureOutput *) inCaptureOutput
   didOutputVideoFrame: (CVImageBufferRef) inVideoFrame 
      withSampleBuffer: (QTSampleBuffer *) inSampleBuffer
        fromConnection: (QTCaptureConnection *) inConnection
{
	_frameRecieved = YES;
	
    // Store the latest frame
	// This must be done in a @synchronized block because this delegate method is not called on the main thread
    CVImageBufferRef	imageBufferToRelease;
	
    CVBufferRetain(inVideoFrame);
    
    @synchronized (self) {
        imageBufferToRelease = _currentImageBuffer;
        _currentImageBuffer = inVideoFrame;
    }
    
    CVBufferRelease(imageBufferToRelease);
}

- (IBAction)changeFilter:(id)sender
{
    NSButton *b = (NSButton *)sender;
    currentFilterType = (int)b.tag;
}

- (CIImage *)flipImage:(CIImage *)image
{
    CIImage *i = [image imageByApplyingTransform:CGAffineTransformMakeScale(-1, 1)];
    i = [i imageByApplyingTransform:CGAffineTransformMakeTranslation([image extent].size.width, 0)];
    return i;
}

- (CIImage *)applyFilterWithImage:(CIImage *)i
{        
    switch (currentFilterType) 
    {
        case IrisFitlerTypeBW:
            currentFilter = [CIFilter filterWithName:@"CIColorMonochrome"];  
            [currentFilter setDefaults];
            [currentFilter setValue:[CIColor colorWithRed:0.75 green:0.75 blue:0.75] forKey:@"inputColor"];
            [currentFilter setValue:[NSNumber numberWithFloat:1.0] forKey:@"inputIntensity"];
            break;
        case IrisFilterTypeSepia:
            currentFilter = [CIFilter filterWithName:@"CISepiaTone"];  
            [currentFilter setDefaults];
            [currentFilter setValue:[NSNumber numberWithFloat:1.0] forKey:@"inputIntensity"];
        default:
            break;
        }
    [currentFilter setValue: i forKey: @"inputImage"];
    i = [currentFilter valueForKey: @"outputImage"];
    return i;
}

@end
