//
//  SnapPreferencesVC.m
//  Snap!
//
//  Created by Brandon Schlenker on 5/17/11.
//  Copyright 2011 Brandon Schlenker. All rights reserved.
//

#import "SnapPreferencesVC.h"

@interface SnapPreferencesVC ()
- (void)buildNamesToDevicesDictionaryForMediaType: (NSString *)mediaType
                                includeMuxDevices:(BOOL)includeMux;
@end

@implementation SnapPreferencesVC
@synthesize accountButton, flipCheckBox, delegate, cameraPicker;

- (id)initWithNibName:(NSString *)nibNameOrNil bundle:(NSBundle *)nibBundleOrNil
{
    self = [super initWithNibName:nibNameOrNil bundle:nibBundleOrNil];
    if (self) {
        
      api = [[DailyBooth alloc] init];
        BOOL b = [[NSUserDefaults standardUserDefaults] boolForKey:@"prefFlipPhotos"];
        if (b)
        {
            [self.flipCheckBox setState:NSOnState];
        }
        else
        {
            [self.flipCheckBox setState:NSOffState];  
        }
    }
    return self;
}

- (void)showPreferences
{
    [self buildNamesToDevicesDictionaryForMediaType:QTMediaTypeVideo includeMuxDevices:YES];
}

- (void)buildNamesToDevicesDictionaryForMediaType: (NSString *)mediaType
                                includeMuxDevices:(BOOL)includeMux 
{
	if (cameraDevices == nil) {
		cameraDevices = [[NSMutableArray alloc] init];
	}
    [cameraPicker removeAllItems];
	[cameraDevices removeAllObjects];
    

	// then find the rest
	NSArray* devicesWithMediaType =
    [QTCaptureDevice inputDevicesWithMediaType:mediaType];
	NSArray* devicesWithMuxType =
    [QTCaptureDevice inputDevicesWithMediaType:QTMediaTypeMuxed];
	NSMutableSet* devicesSet = [NSMutableSet setWithArray: devicesWithMediaType];
	[devicesSet addObjectsFromArray: devicesWithMuxType];
    
	// add all devices from set to dictionary
	NSEnumerator *enumerator = [devicesSet objectEnumerator];
	id value;
	while ((value = [enumerator nextObject])) {
		QTCaptureDevice *device = (QTCaptureDevice*) value;
		
        NSDictionary *deviceDictionary = [NSDictionary dictionaryWithObjectsAndKeys:device,[device localizedDisplayName], [device uniqueID], @"ID", nil];
        [cameraDevices addObject:deviceDictionary];
        
        [cameraPicker insertItemWithTitle:[device localizedDisplayName] atIndex:[cameraDevices indexOfObject:deviceDictionary]];
        
        NSString *selected = [[NSUserDefaults standardUserDefaults] stringForKey:@"IRIS_SELECTED_CAMERA"];
        
        if (selected)
        {
            QTCaptureDevice *d = [QTCaptureDevice deviceWithUniqueID:selected];
            [cameraPicker selectItemWithTitle:[d localizedDisplayName]];
        }
        else
        {
            [cameraPicker selectItemAtIndex:0];
        }
	}
    //NSLog(@"%@",cameraDevices);
}

- (IBAction)selectNewCamera:(id)sender
{
    NSInteger index = [cameraPicker indexOfSelectedItem];
    NSString *ID = [[cameraDevices objectAtIndex:index] objectForKey:@"ID"];
    
    [[NSUserDefaults standardUserDefaults] setObject:ID forKey:@"IRIS_SELECTED_CAMERA"];
    
    NSLog(@"selected camera %@",ID);
    
    [[NSNotificationCenter defaultCenter] postNotificationName:@"IRIS_SELECTED_NEW_CAMERA" object:nil];
}

- (void)checkAuth
{    
    if ([api isAuthenticated])
    {
        [accountButton setTitle:@"Log Out of DailyBooth"];
    }
    else
    {
        [accountButton setTitle:@"Log In to DailyBooth"];
    }  
}

- (IBAction)exitPreferences:(id)sender
{
    [delegate retakePhoto:self];
}

- (IBAction)toggleAccount:(id)sender
{
    NSString *token = [[NSUserDefaults standardUserDefaults] objectForKey:@"oauth_token"];
    NSButton *b = (NSButton *)sender;
    
    if (token)
    {
        [api logout];
        [b setTitle:@"Log In to DailyBooth"];
    }
    
    else
    {
        [api authenticate];
    }
}

- (IBAction)quitApp:(id)sender
{
    [[NSApplication sharedApplication] terminate:sender];
}


- (IBAction)openAbout:(id)sender
{
    [[NSApplication sharedApplication] orderFrontStandardAboutPanel:sender];
}
@end
