//
//  Snap_AppDelegate.m
//  Snap!
//
//  Created by Brandon Schlenker on 5/7/11.
//  Copyright 2011 Brandon Schlenker. All rights reserved.
//

#import "Snap_AppDelegate.h"
#import "DailyBooth.h"

@interface Snap_AppDelegate ()
- (void)registerAppURL;
- (void)toggleWindow:(id)sender;
- (void)closePopover;
@end


@implementation Snap_AppDelegate
@synthesize photoWindow;

- (void)applicationDidFinishLaunching:(NSNotification *)aNotification
{
    float width = 25.0;
    float height = [[NSStatusBar systemStatusBar] thickness];
    NSRect viewFrame = NSMakeRect(0, 0, width, height);
    
    if (!statusItem)
    {
     //   NSLog(@"create status item");
        statusItem = [[[NSStatusBar systemStatusBar] statusItemWithLength:NSVariableStatusItemLength] retain];
        [statusItem setView:[[[SnapCustomStatusItemView alloc] initWithFrame:viewFrame controller:self] autorelease]]; 
        [statusItem setHighlightMode:YES];
    }
    
    dailyBoothAPI = [[DailyBooth alloc] init];
    
    snapCameraVC = [[SnapCameraVC alloc] initWithNibName:@"SnapCameraVC" bundle:[NSBundle mainBundle]];
    snapCameraVC.delegate = self;
    snapCameraVC.DBApi = dailyBoothAPI;
    
    snapUploadVC = [[SnapUploadVC alloc] initWithNibName:@"SnapUploadVC" bundle:[NSBundle mainBundle]];
    snapUploadVC.delegate = self;
    snapUploadVC.DBApi = dailyBoothAPI;
    
    popover = [[INPopoverController alloc] initWithContentViewController:snapCameraVC];
    popover.closesWhenPopoverResignsKey = YES;
    popover.closesWhenApplicationBecomesInactive = YES;
    popover.delegate = self;
    popover.color = [NSColor colorWithCalibratedWhite:1.0 alpha:1];
    popover.borderWidth = 0;

    
    [self registerAppURL];
    [self performSelector:@selector(toggleWindow:) withObject:[statusItem view] afterDelay:.5];
    
    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(shouldPrepUploadOfImage:) name:@"SnapDidDragImage" object:nil];
    
     [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(didLogin:) name:@"DAILYBOOTH_DID_LOGIN" object:nil];
}

- (void)awakeFromNib
{

}

- (void)registerAppURL
{
	[[NSAppleEventManager sharedAppleEventManager] setEventHandler:self andSelector:@selector(getUrl:withReplyEvent:) forEventClass:kInternetEventClass andEventID:kAEGetURL];
}

- (void)getUrl:(NSAppleEventDescriptor *)event withReplyEvent:(NSAppleEventDescriptor *)replyEvent
{
	NSString *url = [[event paramDescriptorForKeyword:keyDirectObject] stringValue];

 //   NSLog(@"open app %@", url);
    NSArray *a = [url componentsSeparatedByString:@"code="];
    if ([a count])
    {
        [dailyBoothAPI getTokenWithCode:[a objectAtIndex:1]];
        [self toggleWindow:[statusItem view]];
    }
}

- (void)toggleWindow:(id)sender
{
    float width = 25.0;
    NSPoint p = NSMakePoint(width/2, 2);
    
    float height = [[NSStatusBar systemStatusBar] thickness];
    NSRect viewFrame = NSMakeRect(0, 0, width, height);
    
    Class popoverClass = NSClassFromString(@"NSPopover--");
    if (popoverClass != nil) 
    {

      /*  if (!lionPopover.isShown)
        {
            
            [lionPopover showRelativeToRect:viewFrame ofView:sender preferredEdge:1];
            NSLog(@"open NSPopover");
        }
        else
        {
            [lionPopover close];
             NSLog(@"close NSPopover");
        } */
    }
    else
    {
        if (!popover.popoverIsVisible)
        {
            [popover showPopoverAtPoint:p inView:sender preferredArrowDirection:INPopoverArrowDirectionUp anchorsToPositionView:YES]; 
            [[NSApplication sharedApplication] activateIgnoringOtherApps:YES];
        }
        else
        {
            [self closePopover];
        }
    }
}

- (void)closePopover
{
    
    if (popover)
    {
        [popover closePopover:self];
    }
    else
    {
     //   [lionPopover close];
    }
    
    [[NSApplication sharedApplication] deactivate];
}

- (void)popoverWillShow:(INPopoverController *)pop
{
    if ([pop isKindOfClass:[INPopoverController class]])
    {
        pop = (INPopoverController *)pop;
    }
    else if ([pop isKindOfClass:[NSNotification class]])
    {
        pop = [pop object];
    }
    
    if (shouldReset)
    {
        if ([[popover popoverWindow] contentView] != snapCameraVC.view)
        {
            [self retakePhoto:self];
            shouldReset = NO;
        }
    }
}

- (void)popoverDidShow:(id)pop
{
    if ([pop isKindOfClass:[INPopoverController class]])
    {
        pop = (INPopoverController *)pop;
    }
    else if ([pop isKindOfClass:[NSNotification class]])
    {
        pop = [pop object];
    }
    
    if ([[pop popoverWindow] contentView] == snapCameraVC.view)
    {
        [snapCameraVC startCamera];
    }
}

- (void)popoverWillClose:(INPopoverController*)pop
{
    if (snapCameraVC.busy)
    {
        [snapCameraVC stopCamera];
        [snapCameraVC.captureView setImage:nil];
    }
}

- (void)snapCameraVC:(SnapCameraVC *)cameraVC didTakePicture:(NSImage *)picture
{
    id pop;
    
    if (popover)
    {
        pop = popover;
    }
    else
    {
     //   pop = lionPopover;
    }
    
    if ([[pop popoverWindow] contentView] == snapCameraVC.view)
    {
         [pop setContentSize:NSMakeSize(267,328)];
        [popover.popoverWindow setContentView:snapUploadVC.view];

        snapUploadVC.imageView.image = picture;
        [snapUploadVC setupLayerStyling];
       [[snapUploadVC.commentTextField window] makeFirstResponder:snapUploadVC.commentTextField];
        [snapUploadVC.commentTextField setSelectedRange:NSMakeRange(0, 1000000)];
    }
}

- (void)snapUploadVC:(SnapUploadVC *)uploadVC didSendPicture:(NSImage *)picture
{
    [self closePopover];

    [(SnapCustomStatusItemView *)statusItem.view setLoading:YES];

}

- (void)snapUploadVC:(SnapUploadVC *)uploadVC didFinishUploadWithResult:(NSString *)result
{
    [(SnapCustomStatusItemView *)statusItem.view setSuccess:YES];
    NSSound *sound = [NSSound soundNamed:@"Glass"];
	[sound play];

    if (popover.popoverIsVisible)
    {
        [self retakePhoto:self];
    }
    else
    {
        shouldReset = YES;
    }
    [self performSelector:@selector(resetStatusItem) withObject:nil afterDelay:3];
}


- (void)snapUploadVC:(SnapUploadVC *)uploadVC didFailUploadWithResult:(NSError *)result
{
    [(SnapCustomStatusItemView *)statusItem.view setSuccess:NO];
    [self toggleWindow:[statusItem view]];
    [self performSelector:@selector(resetStatusItem) withObject:nil afterDelay:3];
    NSSound *sound = [NSSound soundNamed:@"Basso"];
	[sound play];
    NSAlert *alert = [NSAlert alertWithError:result];
    [alert addButtonWithTitle:@"Okay"];
    [alert setAlertStyle:NSWarningAlertStyle];
    [alert runModal];
}

- (void)resetStatusItem
{
    [(SnapCustomStatusItemView *)statusItem.view setLoading:NO];
}

- (void)snapStatusItemView:(SnapCustomStatusItemView *)itemView draggedImage:(NSImage *)image;
{
    id pop;
    
    if (popover)
    {
        pop = popover;
        if (![pop popoverIsVisible])
        {
            [self toggleWindow:[statusItem view]]; 
            [self retakePhoto:nil];
        }
    }
    else
    {
      /*  pop = lionPopover;
        if (![pop shown])
        {
            [self toggleWindow:[statusItem view]];
            [self retakePhoto:nil];
        }*/
    }
    
   
    [self snapCameraVC:nil didTakePicture:image];

}

- (void)retakePhoto:(id)sender
{
    id pop;
    
    if (popover)
    {
        pop = popover;
    }
    else
    {
     //   pop = lionPopover;
    }
   // [popover setContentSize:CGSizeMake(267, 223)];
    if ([popover.popoverWindow contentView] != snapCameraVC.view)
    {
        [pop setContentSize:NSMakeSize(267,242)];
        [snapCameraVC performSelector:@selector(startCamera) withObject:nil afterDelay:.1];
        [popover.popoverWindow setContentView:snapCameraVC.view];
        [snapCameraVC setupLayerStyling];
        snapUploadVC.imageView.image = nil;
    }
}

- (IBAction)openPreferences:(id)sender
{
    id pop;
    
    if (popover)
    {
        pop = popover;
    }
    else
    {
     //   pop = lionPopover;
    }
    
    if ([pop contentViewController] == snapCameraVC)
    {
      //  [snapCameraVC stopCamera];
    }
    if (!preferencesController)
    {
        preferencesController = [[SnapPreferencesVC alloc] initWithNibName:@"SnapPreferencesVC" bundle:[NSBundle mainBundle]];
    }

    
    [[popover popoverWindow] setContentView:preferencesController.view];
    [pop setContentSize:NSMakeSize(267,214)];

    [preferencesController checkAuth];
    [preferencesController showPreferences];
    preferencesController.delegate = self;
}

- (void)didLogin:(NSNotification *)notif
{
    [self retakePhoto:self];
}

- (void)applicationDidResignActive:(NSNotification *)aNotification
{
    do {
        [self closePopover];
    } while (snapCameraVC.busy);
    
}

@end
